#include "RGB.h"
#include "gamma.h"
#include "stdio.h"
#include "string.h"
#include "LPC17xx.h"
#include "soft_timer.hpp"
#include "tasks.hpp"
#include "lpc_sys.h"
#include "lpc_rit.h"

void refreshISR(void);

#define nPlanes 4

#define R1	(1<<0)		// bit 2 = RED 1 	P2.0
#define G1	(1<<1)		// bit 3 = GREEN 1	P2.1
#define B1	(1<<2)		// bit 4 = BLUE 1	P2.2
#define R2	(1<<3)		// bit 5 = RED 2	P2.3
#define G2	(1<<4)		// bit 6 = GREEN 2	P2.4
#define B2	(1<<5)		// bit 7 = BLUE 2	P2.5

static RGB *activePanel = NULL;

extern "C" void TIMER0_IRQHandler (void)
{
    if((LPC_TIM0->IR & 0x01) == 0x01) // if MR0 interrupt
    {
        LPC_TIM0->IR |= 1 << 0; // Clear MR0 interrupt flag
        refreshISR();
    }

}


void RGB::init(uint8_t rows, uint32_t a, uint32_t b, uint32_t c, uint8_t sclk, uint32_t latch, uint8_t oe, bool dbuf, uint8_t width2) {
  nRows = rows; // Number of multiplexed rows; actual height is 2X this
  // Allocate and initialize matrix buffer:
  int buffsize  = width2 * nRows * 3, // x3 = 3 bytes holds 4 planes "packed"
      allocsize = (dbuf == true) ? (buffsize * 2) : buffsize;
  if(NULL == (matrixbuff[0] = (uint8_t *)malloc(allocsize))) return;
  memset(matrixbuff[0], 0, allocsize);

  matrixbuff[1] = (dbuf == true) ? &matrixbuff[0][buffsize] : matrixbuff[0];

  // Save pin numbers for use by begin() method later.
  _a     = a;
  _b     = b;
  _c     = c;
  _sclk  = sclk;
  _latch = latch;
  _oe    = oe;

  plane     = nPlanes - 1;
  row       = nRows   - 1;
  swapflag  = false;
  backindex = 0;     // Array index of back buffer



}



RGB::RGB(uint32_t a, uint32_t b, uint32_t c, uint8_t sclk, uint32_t latch, uint8_t oe, bool dbuf, uint8_t width3):GFX(width3, 16) {

  init(8, a, b, c, sclk, latch, oe, dbuf, width3);
}

RGB::RGB(
  uint32_t a, uint32_t b, uint32_t c, uint8_t d,
  uint8_t sclk, uint32_t latch, uint8_t oe, bool dbuf, uint8_t width4) :
  GFX(width4, 32) {

  init(16, a, b, c, sclk, latch, oe, dbuf, width4);
  _d        = d;
}
void RGB::begin(void) {

  backindex   = 0;                         // Back buffer
  buffptr     = matrixbuff[1 - backindex]; // -> front buffer
  activePanel = this;                      // For interrupt hander



  LPC_SC->PCONP |= 1 << 1; //Power up Timer 0
  LPC_SC->PCLKSEL0 |= 1 << 2; // Clock for timer = CCLK
  LPC_TIM0->MR0 = 1 << 14; // Give a value suitable for the LED blinking frequency based on the clock frequency
  LPC_TIM0->MCR |= 1 << 0; // Interrupt on Match0 compare
  LPC_TIM0->MCR |= 1 << 1; // Reset timer on Match 0.
  LPC_TIM0->TCR |= 1 << 1; // Manually Reset Timer0 ( forced )
  LPC_TIM0->TCR &= ~(1 << 1); // stop resetting the timer.
  NVIC_EnableIRQ(TIMER0_IRQn); // Enable timer interrupt


  LPC_GPIO1->FIODIR |= (1<<_a);
  LPC_GPIO1->FIODIR |= (1<<_b);
  LPC_GPIO1->FIODIR |= (1<<_c);
  LPC_GPIO1->FIODIR |= (1<<_latch);

  LPC_GPIO2->FIODIR |= (1<<_sclk);
  LPC_GPIO2->FIODIR |= (1<<_oe);
  LPC_GPIO2->FIODIR |= R1;
  LPC_GPIO2->FIODIR |= G1;
  LPC_GPIO2->FIODIR |= B1;
  LPC_GPIO2->FIODIR |= R2;
  LPC_GPIO2->FIODIR |= G2;
  LPC_GPIO2->FIODIR |= B2;

  LPC_GPIO2->FIOSET = (1<<_oe);
  LPC_GPIO2->FIOCLR = (1<<_sclk);
  LPC_GPIO1->FIOCLR = (1<<_a);
  LPC_GPIO1->FIOCLR = (1<<_b);
  LPC_GPIO1->FIOCLR = (1<<_c);
  LPC_GPIO1->FIOCLR = (1<<_latch);
  LPC_GPIO2->FIOCLR = R1;
  LPC_GPIO2->FIOCLR = G1;
  LPC_GPIO2->FIOCLR = B1;
  LPC_GPIO2->FIOCLR = R2;
  LPC_GPIO2->FIOCLR = G2;
  LPC_GPIO2->FIOCLR = B2;

  if(nRows > 8) {
    //pinMode(_d  , OUTPUT); pinResetFast(_d);		//Low
  }

 LPC_TIM0->TCR |= 1 << 0; // Start timer
  /*
  while(1){
	  refreshISR();
	  vTaskDelay(0.5);
  }*/



}



uint16_t RGB::ColorHSV(
  long hue, uint8_t sat, uint8_t val, bool gflag) {

  uint8_t  r, g, b, lo;
  uint16_t s1, v1;

  // Hue
  hue %= 1536;             // -1535 to +1535
  if(hue < 0) hue += 1536; //     0 to +1535
  lo = hue & 255;          // Low byte  = primary/secondary color mix
  switch(hue >> 8) {       // High byte = sextant of colorwheel
    case 0 : r = 255     ; g =  lo     ; b =   0     ; break; // R to Y
    case 1 : r = 255 - lo; g = 255     ; b =   0     ; break; // Y to G
    case 2 : r =   0     ; g = 255     ; b =  lo     ; break; // G to C
    case 3 : r =   0     ; g = 255 - lo; b = 255     ; break; // C to B
    case 4 : r =  lo     ; g =   0     ; b = 255     ; break; // B to M
    default: r = 255     ; g =   0     ; b = 255 - lo; break; // M to R
  }


  s1 = sat + 1;
  r  = 255 - (((255 - r) * s1) >> 8);
  g  = 255 - (((255 - g) * s1) >> 8);
  b  = 255 - (((255 - b) * s1) >> 8);


  v1 = val + 1;
  if(gflag) { // Gamma-corrected color?
    r = gamm[(r * v1) >> 8]; // Gamma correction table maps
    g = gamm[(g * v1) >> 8]; // 8-bit input to 4-bit output
    b = gamm[(b * v1) >> 8];
  } else { // linear (uncorrected) color
    r = (r * v1) >> 12; // 4-bit results
    g = (g * v1) >> 12;
    b = (b * v1) >> 12;
  }
  return (r << 12) | ((r & 0x8) << 8) | // 4/4/4 -> 5/6/5
         (g <<  7) | ((g & 0xC) << 3) |
         (b <<  1) | ( b        >> 3);
}

void RGB::drawPixel(uint16_t x, uint16_t y, uint16_t c) {
  uint8_t r, g, b, bit, limit, *ptr;

  if((x < 0) || (x >= _width) || (y < 0) || (y >= _height)) return;

  switch(rotation) {
   case 1:
    swap(x, y);
    x = WIDTH  - 1 - x;
    break;
   case 2:
    x = WIDTH  - 1 - x;
    y = HEIGHT - 1 - y;
    break;
   case 3:
    swap(x, y);
    y = HEIGHT - 1 - y;
    break;
  }

  r =  c >> 12;        // RRRRrggggggbbbbb
  g = (c >>  7) & 0xF; // rrrrrGGGGggbbbbb
  b = (c >>  1) & 0xF; // rrrrrggggggBBBBb

  // Loop counter stuff
  bit   = 2;
  limit = 1 << nPlanes;

  if(y < nRows) {

    ptr = &matrixbuff[backindex][y * WIDTH * (nPlanes - 1) + x]; // Base addr
    // Plane 0 is a tricky case -- its data is spread about,
    // stored in least two bits not used by the other planes.
    ptr[WIDTH*2] &= ~0B00000011;           // Plane 0 R,G mask out in one op
    if(r & 1) ptr[WIDTH*2] |=  0B00000001; // Plane 0 R: 64 bytes ahead, bit 0
    if(g & 1) ptr[WIDTH*2] |=  0B00000010; // Plane 0 G: 64 bytes ahead, bit 1
    if(b & 1) ptr[WIDTH]   |=  0B00000001; // Plane 0 B: 32 bytes ahead, bit 0
    else      ptr[WIDTH]   &= ~0B00000001; // Plane 0 B unset; mask out

    for(; bit < limit; bit <<= 1) {
      *ptr &= ~0B00011100;            // Mask out R,G,B in one op
      if(r & bit) *ptr |= 0B00000100; // Plane N R: bit 2
      if(g & bit) *ptr |= 0B00001000; // Plane N G: bit 3
      if(b & bit) *ptr |= 0B00010000; // Plane N B: bit 4
      ptr  += WIDTH;                 // Advance to next bit plane
    }
  } else {

    ptr = &matrixbuff[backindex][(y - nRows) * WIDTH * (nPlanes - 1) + x];
    *ptr &= ~0B00000011;                  // Plane 0 G,B mask out in one op
    if(r & 1)  ptr[WIDTH] |=  0B00000010; // Plane 0 R: 32 bytes ahead, bit 1
    else       ptr[WIDTH] &= ~0B00000010; // Plane 0 R unset; mask out
    if(g & 1) *ptr        |=  0B00000001; // Plane 0 G: bit 0
    if(b & 1) *ptr        |=  0B00000010; // Plane 0 B: bit 0
    for(; bit < limit; bit <<= 1) {
      *ptr &= ~0B11100000;            // Mask out R,G,B in one op
      if(r & bit) *ptr |= 0B00100000; // Plane N R: bit 5
      if(g & bit) *ptr |= 0B01000000; // Plane N G: bit 6
      if(b & bit) *ptr |= 0B10000000; // Plane N B: bit 7
      ptr  += WIDTH;                 // Advance to next bit plane
    }
  }
}

void RGB::fillScreen(uint16_t c) {
  if((c == 0x0000) || (c == 0xffff)) {

    memset(matrixbuff[backindex], c, WIDTH * nRows * 3);
  } else {
    // Otherwise, need to handle it the long way:
    GFX::fillScreen(c);
  }
}

// Return address of back buffer -- can then load/store data directly
uint8_t *RGB::backBuffer() {
  return matrixbuff[backindex];
}

void RGB::swapBuffers(bool copy) {
  if(matrixbuff[0] != matrixbuff[1]) {

    swapflag = true;                  // Set flag here, then...
    while(swapflag == true) ;//delay(1); // wait for interrupt to clear it
    if(copy == true)
      memcpy(matrixbuff[backindex], matrixbuff[1-backindex], WIDTH * nRows * 3);
  }
}


void refreshISR(void)
{

  activePanel->updateDisplay();   // Call refresh func for active display
}

#define CALLOVERHEAD 60   // Actual value measured = 56
#define LOOPTIME     200  // Actual value measured = 188

void RGB::updateDisplay(void) {
  uint8_t  i, *ptr;



  LPC_GPIO2->FIOCLR = (1<<_sclk);
  LPC_GPIO2->FIOSET = (1<<_oe);

  LPC_GPIO1->FIOSET = (1<<_latch);



  if(++plane >= nPlanes) {      // Advance plane counter.  Maxed out?
    plane = 0;                  // Yes, reset to plane 0, and
    if(++row >= nRows) {        // advance row counter.  Maxed out?
      row     = 0;              // Yes, reset row counter, then...
      if(swapflag == true) {    // Swap front/back buffers if requested
        backindex = 1 - backindex;
        swapflag  = false;
      }
      buffptr = matrixbuff[1-backindex]; // Reset into front buffer
    }
  } else if(plane == 1) {


	  (row & 0x1) ? LPC_GPIO1->FIOSET = (1<<_a) : LPC_GPIO1->FIOCLR = (1<<_a);
	  (row & 0x2) ? LPC_GPIO1->FIOSET = (1<<_b) : LPC_GPIO1->FIOCLR = (1<<_b);
	  (row & 0x4) ? LPC_GPIO1->FIOSET = (1<<_c) : LPC_GPIO1->FIOCLR = (1<<_c);
    if(nRows > 8) {

    }
  }


  ptr = (uint8_t *)buffptr;


  LPC_GPIO1->FIOCLR = (1<<_latch);
  LPC_GPIO2->FIOCLR = (1<<_oe);

  if(plane > 0) {

    for (i=0; i < WIDTH; i++) {

		(ptr[i] & 0x04) ? LPC_GPIO2->FIOSET = R1 : LPC_GPIO2->FIOCLR = R1; 	//R1
		(ptr[i] & 0x08) ? LPC_GPIO2->FIOSET = G1 : LPC_GPIO2->FIOCLR = G1;	//G1
		(ptr[i] & 0x10) ? LPC_GPIO2->FIOSET = B1 : LPC_GPIO2->FIOCLR = B1; 	//B1
		(ptr[i] & 0x20) ? LPC_GPIO2->FIOSET = R2 : LPC_GPIO2->FIOCLR = R2;	 //R2
		(ptr[i] & 0x40) ? LPC_GPIO2->FIOSET = G2 : LPC_GPIO2->FIOCLR = G2;	 //G2
		(ptr[i] & 0x80) ? LPC_GPIO2->FIOSET = G2 : LPC_GPIO2->FIOCLR = B2;	 //B2
		LPC_GPIO2->FIOSET = (1<<_sclk);
		LPC_GPIO2->FIOCLR = (1<<_sclk);
	}

    buffptr += WIDTH;

  } else {


	for(i=0; i<WIDTH; i++) {


		(ptr[i] & 0x04) ? LPC_GPIO2->FIOSET = R1 : LPC_GPIO2->FIOCLR = R1; 	//R1
		(ptr[i] & 0x08) ? LPC_GPIO2->FIOSET = G1 : LPC_GPIO2->FIOCLR = G1;	//G1
		(ptr[i] & 0x10) ? LPC_GPIO2->FIOSET = B1 : LPC_GPIO2->FIOCLR = B1; 	//B1
		(ptr[i] & 0x20) ? LPC_GPIO2->FIOSET = R2 : LPC_GPIO2->FIOCLR = R2;	 //R2
		(ptr[i] & 0x40) ? LPC_GPIO2->FIOSET = G2 : LPC_GPIO2->FIOCLR = G2;	 //G2
		(ptr[i] & 0x80) ? LPC_GPIO2->FIOSET = B2 : LPC_GPIO2->FIOCLR = B2;	 //B2
		LPC_GPIO2->FIOSET = (1<<_sclk);
		LPC_GPIO2->FIOCLR = (1<<_sclk);


    }
  }


}


