/*
 *     SocialLedge.com - Copyright (C) 2013
 *
 *     This file is part of free software framework for embedded processors.
 *     You can use it and/or distribute it as long as this copyright header
 *     remains unmodified.  The code is free for personal use and requires
 *     permission to use in a commercial product.
 *
 *      THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 *      OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 *      MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 *      I SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
 *      CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *     You can reach the author of this software at :
 *          p r e e t . w i k i @ g m a i l . c o m
 */

/**
 * @file
 * @brief This is the application entry point.
 * 			FreeRTOS and stdio printf is pre-configured to use uart0_min.h before main() enters.
 * 			@see L0_LowLevel/lpc_sys.h if you wish to override printf/scanf functions.
 *
 */
#include "tasks.hpp"
#include "examples/examples.hpp"
#include "LPC17xx.h"
#include "led/CMPE244/mfGFX.h"   // Core graphics library
#include "led/CMPE244/RGB.h"
#include "led/CMPE244/paneltext.h"
#include "math.h"
#include "stdio.h"
#include "string.h"
#include "timers.h"

	#define CLK 6
	#define OE  7
	#define LAT 22
	#define A   29
	#define B   28
	#define C   23

volatile uint8_t k = 0;
RGB matt(A, B, C, CLK, LAT, OE, false,128);
const char str[] = "abcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcabcaabcabcabcabcaabcabcabcabcabcabcabcabcabcabcabcabcaabcabcabcabcaabcabcabcabcabcabcabcabcabcabcabcabcaabcabcabcabcaabcabcabcabcabcabcabcaaaaaaz";
char display_text[260]={0};
volatile signed int  textX = matt.width();
volatile signed int  textMin = (128 * -10)-128;
//volatile signed int  textMin = sizeof(display_text) * -3;
volatile uint16_t flag=0;
volatile uint16_t recv_length=0;
volatile uint16_t str_length=0;
volatile uint16_t displaylength=0;
volatile uint8_t pilength=0;
char uart_text[260] = {0};

//char panel_text[128];
volatile uint16_t text_length = 0;
uint32_t  hue = 0;
volatile uint16_t x =0;

class gpioTask : public scheduler_task
 {
 	 public:
	 gpioTask (uint8_t priority):
	 scheduler_task("gpioTask2", 2000,priority)
 	 {

 	 }

	 bool init(void)
	 {
	  LPC_PINCON ->PINSEL9|=(1<<25);
	  LPC_PINCON ->PINSEL9|=(1<<24);

	  LPC_PINCON ->PINSEL9|=(1<<27);
	  LPC_PINCON ->PINSEL9|=(1<<26);

	  LPC_SC ->PCONP|=(1<<25); //UART3 powerup
	  LPC_SC ->PCLKSEL1 |= (1<<18);
	  LPC_SC ->PCLKSEL1 &=~(1<<19); //PCLK ==CCLK
	  //LPC_SC ->PCLKSEL1 |=(1<<19);// clock setting - 10
	  //LPC_SC ->PCLKSEL1 &=~(1<<19);// clock setting - 00 -clk/4==12
	  LPC_UART3->LCR |=(1<<7);///dlab

	  //LPC_UART3->DLL = 113;//baud 2400
	  //LPC_UART3->DLL = 4;//baud 115384

	  //LPC_UART3->DLL = 17;
	  //LPC_UART3->DLM = 0;

	  LPC_UART3->DLL = 40;
	  LPC_UART3->DLM = 0;

	  //LPC_UART3->DLM = 2;//baud 2400
	  //LPC_UART3->DLM = 0;//baud 115384
	  //LPC_UART3->FDR =133;//divaddval=5, mulval=8
	  //LPC_UART3->FDR =248;

	  LPC_UART3->FDR =32;

	  LPC_UART3->LCR &=~(1<<3);//disable parity
	  LPC_UART3->LCR &=~(1<<2);//1 stop bit
	  LPC_UART3->LCR |=(1<<0);//8bit word length
	  LPC_UART3->LCR |=(1<<1);//8bit word length
	  LPC_UART3->FCR |=(1<<0);//fifo enabled
	  LPC_UART3->IER =0;
	  LPC_UART3->ACR =0;
	  LPC_UART3->TER =0x80;//enable transmitter
	  LPC_UART3->LCR &=~(1<<7);///dlab bit set to '0'


	  //configurating switches

	  LPC_PINCON->PINSEL2 &=~(3<<18);//GPIO
	  //LPC_PINCON->PINSEL2 &=~(3<<20);//GPIO

	  LPC_PINCON->PINMODE2 |=(3<<18);
	  //LPC_PINCON->PINMODE2 |=(3<<20);

	  LPC_GPIO1->FIODIR &=~(1<<9);
	  LPC_GPIO1->FIODIR &=~(1<<10);



		 return true;
	 }
	 bool run(void *p)
	 {
			 while(LPC_UART3->LSR & 0x1)
			 {
				 uart_text[recv_length] = LPC_UART3->RBR;
				 if (uart_text[recv_length]=='$') flag = 1;
				 recv_length++;
			 }

			if (flag){
				if(recv_length>255)recv_length = 255;
				 for (int t = 0;t<255;t++){
					 display_text[t]=0;
				 }
				 for (int s = 0;s<(recv_length-1);s++){
					 display_text[s]=uart_text[s];
				 }
				 displaylength=recv_length-1;
				 recv_length = 0;
				 flag = 0;
			}

	 		 return true;
	 	 }

};


void panel(void *p){
	  while(1){

			matt.fillScreen(0);
			matt.setTextColor(matt.ColorHSV(hue, 255, 255, true));
			matt.setCursor(textX, 4);
			//matt.print(paneltext);
			matt.print(display_text);
			//matt.print(str);
			textX -=1;
			hue += 2;
			if(hue >= 1536) hue -= 1536;
			matt.swapBuffers(false);
			vTaskDelay(40);
			//displaylength++1;
			//printf(" %d ",(displaylength/24));
			if((textX) < ( (128 * -((displaylength))/24)-128) ) textX = matt.width();
			//printf(" %d ",textX);
			//printf(" %d ",sizeof(display_text));
			//if (textX == 256 ) textX = 0;//matt.width();

	  }
}




void test(void *p){
	  while(1){
		  printf("%s",paneltexty);
		  vTaskDelay(5000);
	  }
}

void setuppanel(void *p){

	while(1){
	static bool read_sig_flag=false;
	if ( !read_sig_flag ) {
	  matt.begin();
	  matt.setTextWrap(false);
	  matt.setTextSize(1);
	  read_sig_flag = true;
	}
	vTaskDelay(100000);
	}
}



int main(void)
{
	 scheduler_add_task(new gpioTask(PRIORITY_MEDIUM));
	 scheduler_add_task(new terminalTask(PRIORITY_HIGH));
	 xTaskCreate(panel, (const char*)"panel", STACK_BYTES(6144), NULL, PRIORITY_HIGH, 0);
	 xTaskCreate(setuppanel, (const char*)"setuppanel", STACK_BYTES(2048), NULL, PRIORITY_LOW, 0);
	 //xTaskCreate(test, (const char*)"test", STACK_BYTES(2048), NULL, PRIORITY_LOW, 0);



	#if 0
	scheduler_add_task(new periodicSchedulerTask());
	#endif
    /**
     * A few basic tasks for this bare-bone system :
     *      1.  Terminal task provides gateway to interact with the board through UART terminal.
     *      2.  Remote task allows you to use remote control to interact with the board.
     *      3.  Wireless task responsible to receive, retry, and handle mesh network.
     *
     * Disable remote task if you are not using it.  Also, it needs SYS_CFG_ENABLE_TLM
     * such that it can save remote control codes to non-volatile memory.  IR remote
     * control codes can be learned by typing the "learn" terminal command.
     */


    /* Consumes very little CPU, but need highest priority to handle mesh network ACKs */
    scheduler_add_task(new wirelessTask(PRIORITY_CRITICAL));

    /* Change "#if 0" to "#if 1" to run period tasks; @see period_callbacks.cpp */


    /* The task for the IR receiver */
    // scheduler_add_task(new remoteTask  (PRIORITY_LOW));

    /* Your tasks should probably used PRIORITY_MEDIUM or PRIORITY_LOW because you want the terminal
     * task to always be responsive so you can poke around in case something goes wrong.
     */

    /**
     * This is a the board demonstration task that can be used to test the board.
     * This also shows you how to send a wireless packets to other boards.
     */
    #if 0
        scheduler_add_task(new example_io_demo());
    #endif

    /**
     * Change "#if 0" to "#if 1" to enable examples.
     * Try these examples one at a time.
     */
    #if 0
        scheduler_add_task(new example_task());
        scheduler_add_task(new example_alarm());
        scheduler_add_task(new example_logger_qset());
        scheduler_add_task(new example_nv_vars());
    #endif

    /**
	 * Try the rx / tx tasks together to see how they queue data to each other.
	 */
    #if 0
        scheduler_add_task(new queue_tx());
        scheduler_add_task(new queue_rx());
    #endif

    /**
     * Another example of shared handles and producer/consumer using a queue.
     * In this example, producer will produce as fast as the consumer can consume.
     */
    #if 0
        scheduler_add_task(new producer());
        scheduler_add_task(new consumer());
    #endif

    /**
     * If you have RN-XV on your board, you can connect to Wifi using this task.
     * This does two things for us:
     *   1.  The task allows us to perform HTTP web requests (@see wifiTask)
     *   2.  Terminal task can accept commands from TCP/IP through Wifly module.
     *
     * To add terminal command channel, add this at terminal.cpp :: taskEntry() function:
     * @code
     *     // Assuming Wifly is on Uart3
     *     addCommandChannel(Uart3::getInstance(), false);
     * @endcode
     */
    #if 0
        Uart3 &u3 = Uart3::getInstance();
        u3.init(WIFI_BAUD_RATE, WIFI_RXQ_SIZE, WIFI_TXQ_SIZE);
        scheduler_add_task(new wifiTask(Uart3::getInstance(), PRIORITY_LOW));
    #endif

    scheduler_start(); ///< This shouldn't return
    return -1;
}
